--Moonshine Sound Script by PeachyPeach--
----------
-- Data --
----------

sMoonshineAudioData = {
    [0x2C] = { loopStart =  57.31, loopEnd = 137.25, volume = 0.80, filename = "sound/hotel.ogg"       },
	[0x2B] = { loopStart =  89.20, loopEnd = 172, volume = 0.80, filename = "sound/secret.ogg"       },
    [0x0E] = { loopStart =  89.20, loopEnd = 172, volume = 0.80, filename = "sound/shell.ogg"       },
	[0x23] = { loopStart =  0.00, loopEnd = 14.91, volume = 0.80, filename = "sound/hub.ogg"       },
}

sMoonshineBackgroundSeqs = {
    [LEVEL_BOB]    = { [1] = 0x2C },
	[LEVEL_VCUTM]    = { [1] = 0x2B },
	[LEVEL_SA]    = { [1] = 0x2B },
    [LEVEL_BITDW]    = { [1] = 0x2B },
	[LEVEL_CASTLE]    = { [1] = 0x23 },
}

-----------
-- State --
-----------

sMoonshineAudioState = {
    audio = nil,
    seq = 0x00,
    position = 0.0,
    volume = 0.0,
}

function moonshine_get_seq_to_play(level, area)
    if obj_get_first_with_behavior_id(id_bhvActSelector) == nil then
        local seq = sMoonshineBackgroundSeqs[level]
        if seq ~= nil and seq[area] ~= nil then
            return seq[area]
        end
    end
    return 0x00
end

function moonshine_get_audio_target_volume()
    if is_game_paused() then return 0.0 end
    if get_current_background_music() > 0 then return 0.0 end
    if get_current_background_music_target_volume() ~= 0 then return 0.0 end
    if get_current_background_music_max_target_volume() ~= 0 then return 0.0 end
    if is_current_background_music_volume_lowered() ~= 0 then return 0.5 end
    return 1.0
end

------------
-- Update --
-----------

function moonshine_update_audio()
    local m = gMarioStates[0]
    local np = gNetworkPlayers[m.playerIndex]
    local seq = moonshine_get_seq_to_play(np.currLevelNum, np.currAreaIndex)
    local data = sMoonshineAudioData[seq]

    -- Stop the current music
    if sMoonshineAudioState.seq ~= seq then
        if sMoonshineAudioState.audio ~= nil then
            audio_stream_stop(sMoonshineAudioState.audio)
            audio_stream_destroy(sMoonshineAudioState.audio)
        end
        sMoonshineAudioState.audio = nil
        sMoonshineAudioState.seq = 0x00
    end

    -- Start the music to play
    if sMoonshineAudioState.audio == nil and data ~= nil then
        sMoonshineAudioState.audio = audio_stream_load(data.filename)
        audio_stream_play(sMoonshineAudioState.audio, true, data.volume)
        audio_stream_set_looping(sMoonshineAudioState.audio, false)
        sMoonshineAudioState.volume = 1.0
        sMoonshineAudioState.position = 0.0
        sMoonshineAudioState.seq = seq
    end

    -- Update the playing music
    if sMoonshineAudioState.audio ~= nil then

        -- Loop if the music reached the end point
        local pos = audio_stream_get_position(sMoonshineAudioState.audio)
        if pos >= data.loopEnd then
            audio_stream_play(sMoonshineAudioState.audio, true, data.volume)
            audio_stream_set_position(sMoonshineAudioState.audio, data.loopStart)
            sMoonshineAudioState.position = data.loopStart
        end

        -- Update volume
        local targetVolume = moonshine_get_audio_target_volume()
        if sMoonshineAudioState.volume > targetVolume then
            local pirahnaPlant = obj_get_nearest_object_with_behavior_id(m.marioObj, id_bhvPiranhaPlant)
            if pirahnaPlant ~= nil and pirahnaPlant.oAction == PIRANHA_PLANT_ACT_SLEEPING and dist_between_objects(m.marioObj, pirahnaPlant) < 1000 then
                sMoonshineAudioState.volume = math.max(sMoonshineAudioState.volume - 0.02, targetVolume)
            else
                sMoonshineAudioState.volume = math.max(sMoonshineAudioState.volume - 0.10, targetVolume)
            end
            if sMoonshineAudioState.volume == 0.0 then
                sMoonshineAudioState.position = pos
            end
        elseif sMoonshineAudioState.volume < targetVolume then
            if sMoonshineAudioState.volume == 0.0 then
                audio_stream_set_position(sMoonshineAudioState.audio, sMoonshineAudioState.position)
            end
            sMoonshineAudioState.volume = math.min(sMoonshineAudioState.volume + 0.05, targetVolume)
        end
        audio_stream_set_volume(sMoonshineAudioState.audio, data.volume * sMoonshineAudioState.volume)
    end
end

hook_event(HOOK_UPDATE, moonshine_update_audio)

